@@ -4,13 +4,13 @@
 // TODO: #2 share code between the different implementations. There's too much
 //       duplicate code in here.
 
-use std::{
-    io::{Read, Write},
-    ops::Drop,
-};
+use alloc::vec::Vec;
+use core::ops::Drop;
 
 #[cfg(feature = "serde_serialize")]
 use serde::ser::SerializeStruct;
+#[cfg(feature = "std")]
+use std::io::{Read, Write};
 use zeroize::Zeroize;
 
 use crate::{Deserialize, Error, Serialize, Size};
@@ -39,19 +39,34 @@ macro_rules! impl_byte_size {
 
 macro_rules! impl_byte_deserialize {
     ($self:ident, $size:ty, $name:ident, $len_len:literal) => {
+        #[cfg(feature = "std")]
         #[inline(always)]
         fn deserialize_bytes<R: Read>(bytes: &mut R) -> Result<Self, Error> {
             let len = <$size>::tls_deserialize(bytes)? as usize;
+            // When fuzzing we limit the maximum size to allocate.
+            // XXX: We should think about a configurable limit for the allocation
+            //      here.
+            if cfg!(fuzzing) && len > u16::MAX as usize {
+                return Err(Error::DecodingError(format!(
+                    "Trying to allocate {} bytes. Only {} allowed.",
+                    len,
+                    u16::MAX
+                )));
+            }
             let mut result = Self {
                 vec: vec![0u8; len],
             };
             let read = bytes.read(result.vec.as_mut_slice())?;
-            debug_assert_eq!(
-                read, len,
-                "Expected to read {} bytes but {} were read.",
-                len, read
-            );
             if read != len {
+                if !cfg!(fuzzing) {
+                    // When fuzzing we don't want to assert here because it would
+                    // crash all the time. Throwing an error is sufficient.
+                    debug_assert_eq!(
+                        read, len,
+                        "Expected to read {} bytes but {} were read.",
+                        len, read
+                    );
+                }
                 Err(Error::DecodingError(format!(
                     "{} bytes were read but {} were expected",
                     read, len
@@ -65,6 +80,7 @@ macro_rules! impl_byte_deserialize {
 
 macro_rules! impl_deserialize {
     ($self:ident, $size:ty, $name:ident, $len_len:literal) => {
+        #[cfg(feature = "std")]
         #[inline(always)]
         fn deserialize<R: Read>(bytes: &mut R) -> Result<Self, Error> {
             let mut result = Self { vec: Vec::new() };
@@ -83,6 +99,7 @@ macro_rules! impl_deserialize {
 
 macro_rules! impl_serialize {
     ($self:ident, $size:ty, $name:ident, $len_len:literal) => {
+        #[cfg(feature = "std")]
         #[inline(always)]
         fn serialize<W: Write>(&$self, writer: &mut W) -> Result<usize, Error> {
             // Get the byte length of the content, make sure it's not too
@@ -126,6 +143,7 @@ macro_rules! impl_serialize {
 
 macro_rules! impl_byte_serialize {
     ($self:ident, $size:ty, $name:ident, $len_len:literal) => {
+        #[cfg(feature = "std")]
         #[inline(always)]
         fn serialize_bytes<W: Write>(&$self, writer: &mut W) -> Result<usize, Error> {
             // Get the byte length of the content, make sure it's not too
@@ -166,54 +184,48 @@ macro_rules! impl_byte_serialize {
 }
 
 macro_rules! impl_tls_vec_codec_generic {
-    ($size:ty, $name:ident, $len_len: literal, $($bounds:ident),*) => {
-        impl<T: $($bounds + )*> Serialize
-            for $name<T>
-        {
+    ($size:ty, $name:ident, $len_len: literal $(, $bounds:ident)*) => {
+        impl<T: $($bounds + )* Serialize> Serialize for $name<T> {
+            #[cfg(feature = "std")]
             fn tls_serialize<W: Write>(&self, writer: &mut W) -> Result<usize, Error> {
                 self.serialize(writer)
             }
         }
 
-        impl<T: $($bounds + )*> Size
-            for $name<T>
-        {
+        impl<T: $($bounds + )* Size> Size for $name<T> {
             #[inline]
             fn tls_serialized_len(&self) -> usize {
                 self.tls_serialized_length()
             }
         }
 
-        impl<T: $($bounds + )*> Serialize
-            for &$name<T>
-        {
+        impl<T: $($bounds + )* Serialize> Serialize for &$name<T> {
+            #[cfg(feature = "std")]
             fn tls_serialize<W: Write>(&self, writer: &mut W) -> Result<usize, Error> {
                 self.serialize(writer)
             }
         }
 
-        impl<T: $($bounds + )*> Size
-            for &$name<T>
-        {
+        impl<T: $($bounds + )* Size> Size for &$name<T> {
             #[inline]
             fn tls_serialized_len(&self) -> usize {
                 self.tls_serialized_length()
             }
         }
 
-        impl<T: $($bounds + )*> Deserialize
-            for $name<T>
-        {
+        impl<T: $($bounds + )* Deserialize> Deserialize for $name<T> {
+            #[cfg(feature = "std")]
             fn tls_deserialize<R: Read>(bytes: &mut R) -> Result<Self, Error> {
                 Self::deserialize(bytes)
             }
         }
-    }
+    };
 }
 
 macro_rules! impl_tls_vec_codec_bytes {
     ($size:ty, $name:ident, $len_len: literal) => {
         impl Serialize for $name {
+            #[cfg(feature = "std")]
             fn tls_serialize<W: Write>(&self, writer: &mut W) -> Result<usize, Error> {
                 self.serialize_bytes(writer)
             }
@@ -227,6 +239,7 @@ macro_rules! impl_tls_vec_codec_bytes {
         }
 
         impl Serialize for &$name {
+            #[cfg(feature = "std")]
             fn tls_serialize<W: Write>(&self, writer: &mut W) -> Result<usize, Error> {
                 self.serialize_bytes(writer)
             }
@@ -240,6 +253,7 @@ macro_rules! impl_tls_vec_codec_bytes {
         }
 
         impl Deserialize for $name {
+            #[cfg(feature = "std")]
             fn tls_deserialize<R: Read>(bytes: &mut R) -> Result<Self, Error> {
                 Self::deserialize_bytes(bytes)
             }
@@ -287,7 +301,7 @@ macro_rules! impl_vec_members {
         /// Get the underlying vector and consume this.
         #[inline]
         pub fn into_vec(mut self) -> Vec<$element_type> {
-            std::mem::take(&mut self.vec)
+            core::mem::take(&mut self.vec)
         }
 
         /// Add an element to this.
@@ -317,7 +331,7 @@ macro_rules! impl_vec_members {
 
         /// Returns an iterator over the slice.
         #[inline]
-        pub fn iter(&self) -> std::slice::Iter<'_, $element_type> {
+        pub fn iter(&self) -> core::slice::Iter<'_, $element_type> {
             self.vec.iter()
         }
 
@@ -339,13 +353,13 @@ macro_rules! impl_vec_members {
 }
 
 macro_rules! impl_tls_vec_generic {
-    ($size:ty, $name:ident, $len_len: literal, $($bounds:ident),*) => {
+    ($size:ty, $name:ident, $len_len: literal $(, $bounds:ident)*) => {
         #[derive(Eq, Debug)]
         pub struct $name<T: $($bounds + )*> {
             vec: Vec<T>,
         }
 
-        impl<T: Clone + $($bounds + )*> Clone for $name<T> {
+        impl<T: $($bounds + )* Clone> Clone for $name<T> {
             fn clone(&self) -> Self {
                 Self::new(self.vec.clone())
             }
@@ -355,14 +369,14 @@ macro_rules! impl_tls_vec_generic {
             impl_vec_members!(T, $len_len);
         }
 
-        impl<T: std::hash::Hash + $($bounds + )*> std::hash::Hash for $name<T> {
+        impl<T: $($bounds + )* core::hash::Hash> core::hash::Hash for $name<T> {
             #[inline]
-            fn hash<H: std::hash::Hasher>(&self, state: &mut H) {
+            fn hash<H: core::hash::Hasher>(&self, state: &mut H) {
                 self.vec.hash(state)
             }
         }
 
-        impl<T: $($bounds + )*> std::ops::Index<usize> for $name<T> {
+        impl<T: $($bounds + )*> core::ops::Index<usize> for $name<T> {
             type Output = T;
 
             #[inline]
@@ -371,66 +385,59 @@ macro_rules! impl_tls_vec_generic {
             }
         }
 
-        impl<T: std::cmp::PartialEq + $($bounds + )*> std::cmp::PartialEq for $name<T> {
+        impl<T: $($bounds + )* core::cmp::PartialEq> core::cmp::PartialEq for $name<T> {
             fn eq(&self, other: &Self) -> bool {
                 self.vec.eq(&other.vec)
             }
         }
 
-        impl<T: $($bounds + )*> std::ops::IndexMut<usize> for $name<T> {
+        impl<T: $($bounds + )*> core::ops::IndexMut<usize> for $name<T> {
             #[inline]
             fn index_mut(&mut self, i: usize) -> &mut Self::Output {
                 self.vec.index_mut(i)
             }
         }
 
-        impl<T: $($bounds + )*> std::borrow::Borrow<[T]> for $name<T> {
+        impl<T: $($bounds + )*> core::borrow::Borrow<[T]> for $name<T> {
             #[inline]
             fn borrow(&self) -> &[T] {
                 &self.vec
             }
         }
 
-        impl<T: $($bounds + )*> std::iter::FromIterator<T> for  $name<T>  {
+        impl<T: $($bounds + )*> core::iter::FromIterator<T> for $name<T> {
             #[inline]
             fn from_iter<I>(iter: I) -> Self
             where
-                I: IntoIterator<Item = T>, {
+                I: IntoIterator<Item = T>,
+            {
                 let vec = Vec::<T>::from_iter(iter);
-                Self{vec}
+                Self { vec }
             }
         }
 
-        impl<T: $($bounds + )*> From<Vec<T>>
-            for $name<T>
-        {
+        impl<T: $($bounds + )*> From<Vec<T>> for $name<T> {
             #[inline]
             fn from(v: Vec<T>) -> Self {
                 Self::new(v)
             }
         }
 
-        impl<T: Clone + $($bounds + )*> From<&[T]>
-            for $name<T>
-        {
+        impl<T: $($bounds + )* Clone> From<&[T]> for $name<T> {
             #[inline]
             fn from(v: &[T]) -> Self {
                 Self::from_slice(v)
             }
         }
 
-        impl<T: $($bounds + )*> From<$name<T>>
-            for Vec<T>
-        {
+        impl<T: $($bounds + )*> From<$name<T>> for Vec<T> {
             #[inline]
             fn from(mut v: $name<T>) -> Self {
-                std::mem::take(&mut v.vec)
+                core::mem::take(&mut v.vec)
             }
         }
 
-        impl<T: $($bounds + )*> Default
-            for $name<T>
-        {
+        impl<T: $($bounds + )*> Default for $name<T> {
             #[inline]
             fn default() -> Self {
                 Self { vec: Vec::new() }
@@ -455,11 +462,7 @@ macro_rules! impl_tls_vec_generic {
         #[cfg(feature = "serde_serialize")]
         impl<'de, T> serde::de::Deserialize<'de> for $name<T>
         where
-            T: Serialize
-                + Deserialize
-                + Size
-                + $($bounds + )*
-                serde::de::Deserialize<'de>,
+            T: $($bounds + )* serde::de::Deserialize<'de>,
         {
             fn deserialize<D>(deserializer: D) -> Result<Self, D::Error>
             where
@@ -481,8 +484,8 @@ macro_rules! impl_tls_vec_generic {
 
                             fn expecting(
                                 &self,
-                                formatter: &mut std::fmt::Formatter,
-                            ) -> std::fmt::Result {
+                                formatter: &mut core::fmt::Formatter,
+                            ) -> core::fmt::Result {
                                 formatter.write_str("`vec`")
                             }
 
@@ -502,19 +505,15 @@ macro_rules! impl_tls_vec_generic {
                 }
 
                 struct TlsVecVisitor<T> {
-                    data: std::marker::PhantomData<T>,
+                    data: core::marker::PhantomData<T>,
                 }
 
                 impl<'de, T> serde::de::Visitor<'de> for TlsVecVisitor<T>
                 where
-                    T: Serialize
-                        + Deserialize
-                        + Size
-                        + $($bounds + )*
-                        serde::de::Deserialize<'de>,
+                    T: $($bounds + )* serde::de::Deserialize<'de>,
                 {
                     type Value = $name<T>;
-                    fn expecting(&self, formatter: &mut std::fmt::Formatter) -> std::fmt::Result {
+                    fn expecting(&self, formatter: &mut core::fmt::Formatter) -> core::fmt::Result {
                         formatter.write_fmt(format_args!("struct {}<T>", stringify!($name)))
                     }
                     fn visit_seq<V>(self, mut seq: V) -> Result<$name<T>, V::Error>
@@ -549,7 +548,7 @@ macro_rules! impl_tls_vec_generic {
                     stringify!($name),
                     &["vec"],
                     TlsVecVisitor {
-                        data: std::marker::PhantomData,
+                        data: core::marker::PhantomData,
                     },
                 )
             }
@@ -568,14 +567,14 @@ macro_rules! impl_tls_vec {
             impl_vec_members!(u8, 1);
         }
 
-        impl std::hash::Hash for $name {
+        impl core::hash::Hash for $name {
             #[inline]
-            fn hash<H: std::hash::Hasher>(&self, state: &mut H) {
+            fn hash<H: core::hash::Hasher>(&self, state: &mut H) {
                 self.vec.hash(state)
             }
         }
 
-        impl std::ops::Index<usize> for $name {
+        impl core::ops::Index<usize> for $name {
             type Output = u8;
 
             #[inline]
@@ -584,27 +583,27 @@ macro_rules! impl_tls_vec {
             }
         }
 
-        impl std::cmp::PartialEq for $name {
+        impl core::cmp::PartialEq for $name {
             fn eq(&self, other: &Self) -> bool {
                 self.vec.eq(&other.vec)
             }
         }
 
-        impl std::ops::IndexMut<usize> for $name {
+        impl core::ops::IndexMut<usize> for $name {
             #[inline]
             fn index_mut(&mut self, i: usize) -> &mut Self::Output {
                 self.vec.index_mut(i)
             }
         }
 
-        impl std::borrow::Borrow<[u8]> for $name {
+        impl core::borrow::Borrow<[u8]> for $name {
             #[inline]
             fn borrow(&self) -> &[u8] {
                 &self.vec
             }
         }
 
-        impl std::iter::FromIterator<u8> for $name {
+        impl core::iter::FromIterator<u8> for $name {
             #[inline]
             fn from_iter<I>(iter: I) -> Self
             where
@@ -632,7 +631,7 @@ macro_rules! impl_tls_vec {
         impl From<$name> for Vec<u8> {
             #[inline]
             fn from(mut v: $name) -> Self {
-                std::mem::take(&mut v.vec)
+                core::mem::take(&mut v.vec)
             }
         }
 
@@ -677,8 +676,8 @@ macro_rules! impl_tls_vec {
 
                             fn expecting(
                                 &self,
-                                formatter: &mut std::fmt::Formatter,
-                            ) -> std::fmt::Result {
+                                formatter: &mut core::fmt::Formatter,
+                            ) -> core::fmt::Result {
                                 formatter.write_str("`vec`")
                             }
 
@@ -698,12 +697,12 @@ macro_rules! impl_tls_vec {
                 }
 
                 struct TlsVecVisitor {
-                    data: std::marker::PhantomData<u8>,
+                    data: core::marker::PhantomData<u8>,
                 }
 
                 impl<'de> serde::de::Visitor<'de> for TlsVecVisitor {
                     type Value = $name;
-                    fn expecting(&self, formatter: &mut std::fmt::Formatter) -> std::fmt::Result {
+                    fn expecting(&self, formatter: &mut core::fmt::Formatter) -> core::fmt::Result {
                         formatter.write_fmt(format_args!("struct {}", stringify!($name)))
                     }
                     fn visit_seq<V>(self, mut seq: V) -> Result<$name, V::Error>
@@ -738,7 +737,7 @@ macro_rules! impl_tls_vec {
                     stringify!($name),
                     &["vec"],
                     TlsVecVisitor {
-                        data: std::marker::PhantomData,
+                        data: core::marker::PhantomData,
                     },
                 )
             }
@@ -748,39 +747,28 @@ macro_rules! impl_tls_vec {
 
 macro_rules! impl_secret_tls_vec {
     ($size:ty, $name:ident, $len_len: literal) => {
-        impl_tls_vec_generic!(
-            $size,
-            $name,
-            $len_len,
-            Serialize,
-            Deserialize,
-            Size,
-            Zeroize
-        );
-        impl_tls_vec_codec_generic!(
-            $size,
-            $name,
-            $len_len,
-            Serialize,
-            Deserialize,
-            Size,
-            Zeroize
-        );
+        impl_tls_vec_generic!($size, $name, $len_len, Zeroize);
+        impl_tls_vec_codec_generic!($size, $name, $len_len, Zeroize);
 
-        impl<T: Serialize + Deserialize + Size + Zeroize> $name<T> {
-            // This implements serialize and size for all versions
+        impl<T: Serialize + Zeroize> $name<T> {
             impl_serialize!(self, $size, $name, $len_len);
+        }
+
+        impl<T: Size + Zeroize> $name<T> {
             impl_size!(self, $size, $name, $len_len);
+        }
+
+        impl<T: Deserialize + Zeroize> $name<T> {
             impl_deserialize!(self, $size, $name, $len_len);
         }
 
-        impl<T: Serialize + Deserialize + Size + Zeroize> Zeroize for $name<T> {
+        impl<T: Zeroize> Zeroize for $name<T> {
             fn zeroize(&mut self) {
                 self.vec.zeroize()
             }
         }
 
-        impl<T: Serialize + Deserialize + Size + Zeroize> Drop for $name<T> {
+        impl<T: Zeroize> Drop for $name<T> {
             fn drop(&mut self) {
                 self.zeroize()
             }
@@ -790,14 +778,19 @@ macro_rules! impl_secret_tls_vec {
 
 macro_rules! impl_public_tls_vec {
     ($size:ty, $name:ident, $len_len: literal) => {
-        impl_tls_vec_generic!($size, $name, $len_len, Serialize, Deserialize, Size);
+        impl_tls_vec_generic!($size, $name, $len_len);
 
-        impl_tls_vec_codec_generic!($size, $name, $len_len, Serialize, Deserialize, Size);
+        impl_tls_vec_codec_generic!($size, $name, $len_len);
 
-        impl<T: Serialize + Deserialize + Size> $name<T> {
-            // This implements serialize and size for all versions
+        impl<T: Serialize> $name<T> {
             impl_serialize!(self, $size, $name, $len_len);
+        }
+
+        impl<T: Size> $name<T> {
             impl_size!(self, $size, $name, $len_len);
+        }
+
+        impl<T: Deserialize> $name<T> {
             impl_deserialize!(self, $size, $name, $len_len);
         }
     };
@@ -840,9 +833,10 @@ macro_rules! impl_tls_byte_slice {
         pub struct $name<'a>(pub &'a [u8]);
 
         impl<'a> $name<'a> {
+            /// Get the raw slice.
             #[inline(always)]
-            fn as_slice(&self) -> &[u8] {
-                &self.0
+            pub fn as_slice(&self) -> &[u8] {
+                self.0
             }
         }
 
@@ -852,12 +846,14 @@ macro_rules! impl_tls_byte_slice {
         }
 
         impl<'a> Serialize for &$name<'a> {
+            #[cfg(feature = "std")]
             fn tls_serialize<W: Write>(&self, writer: &mut W) -> Result<usize, Error> {
                 self.serialize_bytes(writer)
             }
         }
 
         impl<'a> Serialize for $name<'a> {
+            #[cfg(feature = "std")]
             fn tls_serialize<W: Write>(&self, writer: &mut W) -> Result<usize, Error> {
                 self.serialize_bytes(writer)
             }
@@ -885,40 +881,46 @@ impl_tls_byte_slice!(u32, TlsByteSliceU32, 4);
 
 macro_rules! impl_tls_slice {
     ($size:ty, $name:ident, $len_len: literal) => {
-        pub struct $name<'a, T: Size + Serialize>(pub &'a [T]);
+        pub struct $name<'a, T>(pub &'a [T]);
 
-        impl<'a, T: Size + Serialize> $name<'a, T> {
+        impl<'a, T> $name<'a, T> {
+            /// Get the raw slice.
             #[inline(always)]
-            fn as_slice(&self) -> &[T] {
-                &self.0
+            pub fn as_slice(&self) -> &[T] {
+                self.0
             }
         }
 
-        impl<'a, T: Size + Serialize> $name<'a, T> {
-            impl_serialize!(self, $size, $name, $len_len);
+        impl<'a, T: Size> $name<'a, T> {
             impl_size!(self, $size, $name, $len_len);
         }
 
-        impl<'a, T: Size + Serialize> Serialize for &$name<'a, T> {
+        impl<'a, T: Serialize> $name<'a, T> {
+            impl_serialize!(self, $size, $name, $len_len);
+        }
+
+        impl<'a, T: Serialize> Serialize for &$name<'a, T> {
+            #[cfg(feature = "std")]
             fn tls_serialize<W: Write>(&self, writer: &mut W) -> Result<usize, Error> {
                 self.serialize(writer)
             }
         }
 
-        impl<'a, T: Size + Serialize> Serialize for $name<'a, T> {
+        impl<'a, T: Serialize> Serialize for $name<'a, T> {
+            #[cfg(feature = "std")]
             fn tls_serialize<W: Write>(&self, writer: &mut W) -> Result<usize, Error> {
                 self.serialize(writer)
             }
         }
 
-        impl<'a, T: Size + Serialize> Size for &$name<'a, T> {
+        impl<'a, T: Size> Size for &$name<'a, T> {
             #[inline]
             fn tls_serialized_len(&self) -> usize {
                 self.tls_serialized_length()
             }
         }
 
-        impl<'a, T: Size + Serialize> Size for $name<'a, T> {
+        impl<'a, T: Size> Size for $name<'a, T> {
             #[inline]
             fn tls_serialized_len(&self) -> usize {
                 self.tls_serialized_length()
@@ -931,14 +933,14 @@ impl_tls_slice!(u8, TlsSliceU8, 1);
 impl_tls_slice!(u16, TlsSliceU16, 2);
 impl_tls_slice!(u32, TlsSliceU32, 4);
 
-impl From<std::num::TryFromIntError> for Error {
-    fn from(_e: std::num::TryFromIntError) -> Self {
+impl From<core::num::TryFromIntError> for Error {
+    fn from(_e: core::num::TryFromIntError) -> Self {
         Self::InvalidVectorLength
     }
 }
 
-impl From<std::convert::Infallible> for Error {
-    fn from(_e: std::convert::Infallible) -> Self {
+impl From<core::convert::Infallible> for Error {
+    fn from(_e: core::convert::Infallible) -> Self {
         Self::InvalidVectorLength
     }
 }
